@@ -1,7 +1,7 @@
 import platform
 import subprocess
 from os import path, environ
-from typing import Optional
+from typing import Optional, Callable, List, Coroutine, Any
 
 from .connection import start_connection, CustomConnection
 from .objects import *
@@ -48,9 +48,23 @@ class IngeniumAPI:
     _connection: CustomConnection
     _proxy: subprocess.Popen
 
-    def __init__(self):
+    _detected_sock_fn: Optional[Callable[[int, str], None]] = None
+    _detected_socks: List[int] = []
+
+    _state = None
+
+    def __init__(self, state=None):
+        self._state = state
         pass
 
+    def set_sock_detection(self, detected_fn: Callable[[int, str], Coroutine[Any, Any, None]], detected_socks: List[int]):
+        self._detected_sock_fn = detected_fn
+        self._detected_socks = detected_socks
+
+    @property
+    def state(self):
+        return self._state
+
     def remote(self, username: str, password: str):
         self._user = username
         self._pass = password
@@ -104,6 +118,8 @@ class IngeniumAPI:
                 self._objects.append(IngSif(self, self._is_knx, address, ctype, components))
             elif ctype.is_air_sensor():
                 self._objects.append(IngAirSensor(self, self._is_knx, address, ctype, components))
+            elif ctype.is_noise_sensor():
+                self._objects.append(IngNoiseSensor(self, self._is_knx, address, ctype, components))
             elif ctype.is_busing_regulator():
                 self._objects.append(IngBusingRegulator(self, self._is_knx, address, ctype, components))
             elif ctype.is_thermostat():
@@ -113,6 +129,14 @@ class IngeniumAPI:
             self.onload()
         return True
 
+    @property
+    def host(self) -> bool:
+        return self._host
+
+    @property
+    def user(self) -> bool:
+        return self._user
+
     @property
     def is_remote(self) -> bool:
         return self._host is None
@@ -188,6 +212,14 @@ class IngeniumAPI:
                     entities.append((obj, comp, 1))
         return entities
 
+    def get_noise_sensors(self):
+        entities = []
+        for obj in self.objects:
+            if isinstance(obj, IngNoiseSensor):
+                for comp in obj.components:
+                    entities.append((obj, comp))
+        return entities
+
     def get_lights(self):
         entities = []
         for obj in self.objects:
