@@ -37,13 +37,11 @@ class CustomConnection:
 
         for o in self.api.objects:
             # Only allow the command 23 on a Sif
-            if p.source == o.address and isinstance(o, IngSif):
-                print("SIF", o.components[0].id, p)
-
             if p.command == 23 and not isinstance(o, IngSif):
                 return
 
-            if await o.update_state(p):
+            should_update = await o.update_state(p)
+            if should_update:
                 for c in o.components:
                     try:
                         c.update_notify()
