@@ -1,5 +1,6 @@
 import asyncio
 import inspect
+import time
 from enum import Enum
 from typing import Callable, List, TYPE_CHECKING, Dict, Any
 
@@ -38,6 +39,8 @@ class IngComponentType(Enum):
 
     COD_AIR_QUALITY = 8
 
+    COD_NOISE = 9
+
     """
     Actuadores Busing
     """
@@ -85,6 +88,8 @@ class IngComponentType(Enum):
 
     def is_air_sensor(self): return self in [self.COD_AIR_QUALITY]
 
+    def is_noise_sensor(self): return self in [self.COD_NOISE]
+
     def is_actuator(self): return self in [self.COD2E2S, self.COD4E4S, self.COD6E6S, self.COD_KCTR, self.T_KNX_ONOFF]
 
     def is_meterbus(self): return self in [self.COD_METERBUS]
@@ -214,7 +219,6 @@ class IngObject:
 ACTUATOR_BLIND = 0
 ACTUATOR_NORMAL = 2
 
-
 class IngActuator(IngObject):
     def __init__(self, api: 'IngeniumAPI', is_knx: bool, address: int, ctype: IngComponentType,
                  comps: List[IngComponent]):
@@ -230,10 +234,20 @@ class IngActuator(IngObject):
         self.consumption = -1
 
         self.current = -1
+        
         self.voltage = -1
+        self.voltages = []
+        self.last_voltage = -1
+        self.last_voltage_time = 0
+
         self.active_power_bits = [-1, -1]
         self.active_power = -1
 
+        self.is_sock = False
+
+        if self.address in self.api._detected_socks:
+            self.is_sock = True
+
         self.output_state = 0
         self.mode = ACTUATOR_BLIND if self.components[0].icon == 5 else ACTUATOR_NORMAL
 
@@ -306,6 +320,12 @@ class IngActuator(IngObject):
             p = Package(0xFF, self.address, 3, val, 0)
             await self.api.send(p)
 
+    async def maybe_detect_sock(self):
+        if not self.is_sock:
+            self.is_sock = True
+            if self.api._detected_sock_fn is not None:
+                await self.api._detected_sock_fn(self.address, self.components[0].label)
+
     async def update_state(self, p: Package) -> bool:
         # This part is run when receiving the response to a manual read
         if self.address == p.source and p.data1 == p.data2:
@@ -331,17 +351,21 @@ class IngActuator(IngObject):
         if self.mode == ACTUATOR_BLIND:
             for c in self.components:
                 if c.output < 2 and p.data1 == 4:
+                    old_value = self.blind1
                     self.blind1 = p.data2
-                    return True
+                    return old_value != self.blind1
                 elif 2 <= c.output < 4 and p.data1 == 5:
+                    old_value = self.blind2
                     self.blind2 = p.data2
-                    return True
+                    return old_value != self.blind2
                 elif 4 <= c.output < 6 and p.data1 == 6:
+                    old_value = self.blind3
                     self.blind3 = p.data2
-                    return True
+                    return old_value != self.blind3
                 elif 6 <= c.output < 8 and p.data1 == 7:
+                    old_value = self.blind4
                     self.blind4 = p.data2
-                    return True
+                    return old_value != self.blind4
                 return False
 
         else:
@@ -359,27 +383,42 @@ class IngActuator(IngObject):
             elif p.data1 == 3:
                 self.output_state ^= (1 << p.data2)
                 return self.output_state != old_state
+
             elif p.data1 == 86:
                 # In case we get a weird zero when it's not a power meter
-                if self.consumption == -1 and p.data2 == 0:
+                if self.consumption == -1 and self.voltage == -1 and p.data2 == 0:
                     return False
-                factor = 230 / 10
+                await self.maybe_detect_sock()
+                voltage = 230 if self.voltage <= 0 else self.voltage
+                factor = voltage / 10.0
                 old_cons = self.consumption
                 self.consumption = p.data2 * factor
                 self.current = p.data2 / 10.0
                 return self.consumption != old_cons
             elif p.data1 == 87:
+                await self.maybe_detect_sock()
                 # Voltage
-                old_voltage = self.voltage
-                self.voltage = p.data2
-                return self.voltage != old_voltage
+                
+                self.voltages.append(p.data2)  
+                if len(self.voltages) > 5:     
+                    self.voltages.pop(0)
 
+                self.voltage = sum(self.voltages) // len(self.voltages)
+
+                now = int(time.time())
+                if (now - self.last_voltage_time) > 60 and self.voltage != self.last_voltage:
+                    self.last_voltage_time = now
+                    self.last_voltage = self.voltage
+                    return True
+     
+                return False
             elif p.data1 == 88:
+                await self.maybe_detect_sock()
                 old_value = self.active_power_bits[0]
                 self.active_power_bits[0] = p.data2
                 return self.active_power_bits[0] != old_value
-
             elif p.data1 == 89:
+                await self.maybe_detect_sock()
                 old_value = self.active_power_bits[1]
                 self.active_power_bits[1] = p.data2
                 if self.active_power_bits[0] != -1:
@@ -405,6 +444,8 @@ class IngMeterBus(IngObject):
 
         self.updating_channels = []
 
+        self.factor = 1
+
         self.available1: bool = False
         self.available2: bool = False
         self.available3: bool = False
@@ -449,7 +490,7 @@ class IngMeterBus(IngObject):
                 mode = self.updating_channels.pop()
 
                 factor = 230 / 10
-                value = p.data2 * factor
+                value = p.data2 * factor * self.factor
                 available = p.data2 != 255
 
                 if mode not in METERBUS_FIELDS:
@@ -457,7 +498,7 @@ class IngMeterBus(IngObject):
 
                 key, available_key = METERBUS_FIELDS[mode]
                 old_val = getattr(self, key)
-                setattr(self, key, p.data1 * factor)
+                setattr(self, key, value)
 
                 if available_key is not None:
                     setattr(self, available_key, available)
@@ -470,8 +511,12 @@ class IngMeterBus(IngObject):
         if self.address != p.target:
             return False
 
+        if p.data1 == 9: # and p.data2 == 50:
+            self.factor = 2
+            return False
+
         factor = 230 / 10
-        value = p.data2 * factor
+        value = p.data2 * factor * self.factor
         available = p.data2 != 255
 
         if p.data1 not in METERBUS_FIELDS:
@@ -479,7 +524,7 @@ class IngMeterBus(IngObject):
 
         key, available_key = METERBUS_FIELDS[p.data1]
         old_val = getattr(self, key)
-        setattr(self, key, p.data2 * factor)
+        setattr(self, key, value)
 
         if available_key is not None:
             setattr(self, available_key, available)
@@ -552,13 +597,16 @@ class IngSif(IngObject):
         self.values_255[p.data1] = p.data2
 
         if p.command == 23:
-            old_bat_baja = self.bat_baja
             self.bat_baja = True
-            return not old_bat_baja
+            return True
 
         return self.values[p.data1] != old_value
 
     def get_available(self, mode: int):
+        # Don't let temperature get to zero
+        if mode == 0 and self.values[mode] == 0:
+            return False
+
         return self.available and self.values_255[mode] != 255
 
     def get_value(self, mode):
@@ -607,6 +655,47 @@ class IngAirSensor(IngObject):
         return self.thresh_values[mode]
 
 
+class IngNoiseSensor(IngObject):
+    def __init__(self, api: 'IngeniumAPI', is_knx: bool, address: int, ctype: IngComponentType,
+                 comps: List[IngComponent]):
+        super().__init__(api, is_knx, address, ctype, comps)
+
+        self.value = 255
+        self.max = 255
+        self.min = 255
+
+    async def update_state(self, p: Package) -> bool:
+        if self.address != p.target:
+            return False
+
+        if p.data1 == 0:
+            old_value = self.value
+            self.value = p.data2
+            return self.value != old_value
+
+        if p.data1 == 1:
+            self.max = 255
+            return False
+
+        if p.data1 == 2:
+            self.min = 255
+            return False
+
+        return False
+
+    def get_available(self):
+        return self.value != 255 and self.value != 0
+
+    def get_value(self):
+        return self.value
+
+    def get_max(self):
+        return self.max
+
+    def get_min(self):
+        return self.min
+
+
 class IngBusingRegulator(IngObject):
     def __init__(self, api: 'IngeniumAPI', is_knx: bool, address: int, ctype: IngComponentType,
                  comps: List[IngComponent]):
